import { DrawCommand } from "regl";
import { CoordinateSystem } from "./coordinates/coordinate-system";
export declare type UnpackPromise<T> = T extends Promise<infer U> ? U : never;
export declare type Viewport = {
    x: number;
    y: number;
    width: number;
    height: number;
};
export declare enum RenderableType {
    Primitive = 0,
    Composite = 1
}
export declare abstract class Primitive {
    readonly kind = RenderableType.Primitive;
    abstract command(glsl: string): DrawCommand;
    abstract render(command: DrawCommand): void;
    abstract dispose(): void;
}
export declare abstract class Composite {
    readonly kind = RenderableType.Composite;
    abstract children(): Renderable;
    readonly scope: DrawCommand | null;
    props(coords: CoordinateSystem): {};
    dispose(): void;
}
export declare type Renderable = Primitive | Composite | Renderable[];
export declare type NumberArray = number[] | Float32Array;
export declare type Vector2 = NumberArray;
export declare type Vector3 = NumberArray;
export declare type Vector4 = NumberArray;
