import { CoordinateSystem, Kind } from "./coordinate-system";
export function createCartesianCoordinateSystem(cg, xscale, yscale) {
    return new CartesianCoordinateSystem(cg, xscale, yscale);
}
export class CartesianCoordinateSystem extends CoordinateSystem {
    constructor(cg, xscale, yscale) {
        super();
        this.xscale = xscale;
        this.yscale = yscale;
        this.kind = Kind.Cartesian;
        this.scope = cg.regl({
            uniforms: {
                xdomain: cg.regl.prop("xdomain"),
                ydomain: cg.regl.prop("ydomain"),
                xrange: cg.regl.prop("xrange"),
                yrange: cg.regl.prop("yrange"),
            },
        });
        const xglsl = xscale.glsl.replace("toDomain", "toXDomain").replace("toRange", "toXRange");
        const yglsl = yscale.glsl.replace("toDomain", "toYDomain").replace("toRange", "toYRange");
        this.glsl = `
      uniform vec2 xdomain, ydomain;
      uniform vec2 xrange, yrange;

      ${xglsl}
      ${yglsl}

      vec2 toRange(vec2 v) {
        return vec2(
          toXRange(v.x, xdomain, xrange),
          toYRange(v.y, ydomain, yrange)
        );
      }

      vec2 toDomain(vec2 v) {
        return vec2(
          toXDomain(v.x, xdomain, xrange),
          toYDomain(v.y, ydomain, yrange)
        );
      }`;
    }
    toRange(v) {
        return [this.xscale.toRange(v[0]), this.yscale.toRange(v[1])];
    }
    toDomain(v) {
        return [this.xscale.toDomain(v[0]), this.yscale.toDomain(v[1])];
    }
    props() {
        return {
            xdomain: this.xscale.domain,
            ydomain: this.yscale.domain,
            xrange: this.xscale.range,
            yrange: this.yscale.range,
        };
    }
}
//# sourceMappingURL=cartesian.js.map