export function createFont(cg, image, json) {
    return new Font(cg.regl, image, json);
}
export class Font {
    constructor(regl, image, json) {
        this.glyphs = [];
        this.texture = regl.texture({
            data: image,
            mag: "linear",
            min: "linear",
            flipY: true,
        });
        const scale = 1.0 / json.info.size;
        this.lineHeight = scale * json.common.lineHeight;
        const { scaleW, scaleH } = json.common;
        for (const char of json.chars) {
            this.glyphs[char.id] = {
                id: char.id,
                xoffset: scale * char.xoffset,
                yoffset: scale * char.yoffset,
                width: scale * char.width,
                height: scale * char.height,
                xadvance: scale * char.xadvance,
                // Original image is flipped, and the original coordinates are from
                // top left to bottom right, so we need to flip the y-coordinates (1 - y)
                // AND swap their order (y0, y1 = y1, y0) to make things line up normally
                // in webgl-land.
                uv: [
                    char.x / scaleW,
                    1.0 - (char.y + char.height) / scaleH,
                    (char.x + char.width) / scaleW,
                    1.0 - char.y / scaleH,
                ],
            };
        }
        // Create the kerning table for quick lookups.
        this.maxid = this.glyphs.length;
        this.kernTable = new Int8Array(this.maxid * this.maxid);
        for (const kern of json.kernings) {
            this.kernTable[kern.first * this.maxid + kern.second] = scale * kern.amount;
        }
    }
    kern(first, second) {
        return this.kernTable[first * this.maxid + second];
    }
    dispose() {
        this.texture.destroy();
    }
}
//# sourceMappingURL=font.js.map