import { Primitive } from "../common";
import { createDataset } from "./dataset";
const DEFAULT_OPTIONS = {
    colors: [0, 0, 0, 0.5],
    scales: [1, 1],
    rotations: 0,
};
export function createInterleavedShapes(cg, shape, xys, options) {
    return new InterleavedShapes(cg.regl, shape, xys, options);
}
export class InterleavedShapes extends Primitive {
    constructor(regl, shape, xys, options = {}) {
        super();
        this.regl = regl;
        const opts = Object.assign(Object.assign({}, DEFAULT_OPTIONS), options);
        this.shape = createDataset(regl, shape);
        this.xys = createDataset(regl, xys);
        this.scales = createDataset(regl, opts.scales);
        this.rotations = createDataset(regl, opts.rotations);
        this.colors = createDataset(regl, opts.colors);
    }
    command(glsl) {
        return this.regl({
            vert: `
          precision highp float;
          attribute vec2 position;
          attribute vec2 xy, scale;
          attribute float rotation;
          attribute vec4 color;

          varying vec4 vColor;

          ${glsl}

          void main() {
            vec2 pos = scale * position;
            float sint = sin(rotation);
            float cost = cos(rotation);
            pos = vec2(
              cost * pos.x - sint * pos.y,
              sint * pos.x + cost * pos.y
            );
            vec2 screenPosition = toRange(xy) + pos;
            gl_Position = rangeToClip(screenPosition);
            vColor = color;
          }`,
            frag: `
          precision highp float;

          varying vec4 vColor;

          void main() {
            gl_FragColor = vColor;
          }`,
            attributes: {
                position: {
                    buffer: this.regl.prop("position"),
                    divisor: 0,
                },
                xy: {
                    buffer: this.regl.prop("xy"),
                    divisor: 1,
                },
                scale: {
                    buffer: this.regl.prop("scale"),
                    divisor: this.regl.prop("scaleDivisor"),
                },
                rotation: {
                    buffer: this.regl.prop("rotation"),
                    divisor: this.regl.prop("rotationDivisor"),
                },
                color: {
                    buffer: this.regl.prop("color"),
                    divisor: this.regl.prop("colorDivisor"),
                },
            },
            count: this.regl.prop("count"),
            instances: this.regl.prop("instances"),
        });
    }
    render(command) {
        const { shape, xys, scales, rotations, colors } = this;
        const instances = xys.count(2);
        command({
            instances,
            xy: xys.buffer,
            position: shape.buffer,
            scale: scales.buffer,
            rotation: rotations.buffer,
            color: colors.buffer,
            scaleDivisor: scales.divisor(instances, 2),
            rotationDivisor: rotations.divisor(instances, 1),
            colorDivisor: colors.divisor(instances, 4),
            count: shape.count(2),
        });
    }
    dispose() {
        this.xys.dispose();
        this.shape.dispose();
        this.scales.dispose();
        this.rotations.dispose();
        this.colors.dispose();
    }
}
//# sourceMappingURL=interleaved-shapes.js.map