import { Primitive } from "../common";
import { createDataset } from "./dataset";
const DEFAULT_OPTIONS = {
    colors: [0, 0, 0, 0.5],
};
function getPositionBuffer(cg) {
    if (!cg.hasPositionBuffer("rects")) {
        cg.setPositionBuffer("rects", 
        // prettier-ignore
        [0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1]);
    }
    return cg.getPositionBuffer("rects");
}
export function createRects(cg, rects, options) {
    const positionBuffer = getPositionBuffer(cg);
    return new Rects(cg.regl, positionBuffer, rects, options);
}
export class Rects extends Primitive {
    constructor(regl, positionBuffer, rects, options = {}) {
        super();
        this.regl = regl;
        this.positionBuffer = positionBuffer;
        const opts = Object.assign(Object.assign({}, DEFAULT_OPTIONS), options);
        this.rects = createDataset(regl, rects);
        this.colors = createDataset(regl, opts.colors);
    }
    command(glsl) {
        return this.regl({
            vert: `
          precision highp float;
          attribute vec2 position;
          attribute vec4 rect;
          attribute vec4 color;

          varying vec4 vColor;

          ${glsl}

          void main() {
            gl_Position = domainToClip(rect.xy + position.xy * rect.zw);
            vColor = color;
          }`,
            frag: `
          precision highp float;

          varying vec4 vColor;

          void main() {
            gl_FragColor = vColor;
          }`,
            attributes: {
                position: {
                    buffer: this.positionBuffer,
                    divisor: 0,
                },
                rect: {
                    buffer: this.regl.prop("rect"),
                    divisor: 1,
                },
                color: {
                    buffer: this.regl.prop("color"),
                    divisor: this.regl.prop("colorDivisor"),
                },
            },
            count: 6,
            instances: this.regl.prop("instances"),
        });
    }
    render(command) {
        const { rects, colors } = this;
        const instances = rects.count(4);
        command({
            instances,
            rect: rects.buffer,
            color: colors.buffer,
            colorDivisor: colors.divisor(instances, 4),
        });
    }
    dispose() {
        this.rects.dispose();
        this.colors.dispose();
    }
}
//# sourceMappingURL=rects.js.map